/*
 * File   :     tab.java
 * Author :     Tis Veugen
 * Date   :     1998-11-05
 * Version:     1.0
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;

public class tab extends Frame
   implements ActionListener, ItemListener, TextListener
{
  private Button                mInput;
  private Button                mOutput;
  private Button                mTabs;
  private Button                mSpaces;

  private Choice                mTabChoice;

  private TextField             mInputTextField;
  private TextField             mOutputTextField;

  private BufferedReader        mInputFile;
  private PrintWriter           mOutputFile;

  private String                mDirectory = "";
  private int                   mTabSize = 2;

  public                        tab()
  {
    mInputFile  = null;
    mOutputFile = null;

    setTitle( "TAB converter (v1.0)" );

    GridBagLayout               lGbl = new GridBagLayout();
    setLayout( lGbl );

    mInput   = addButton( "Input" );
    mOutput  = addButton( "Output" );
    mTabs    = addButton( "Tabs" );
    mSpaces  = addButton( "Spaces" );

    mInputTextField  = addTextField();
    mOutputTextField = addTextField();

    mTabChoice = addChoice();
    for ( int iTabSize = 2; iTabSize <= 8; iTabSize++ )
    {
      mTabChoice.add( "" + iTabSize );
    }

    GridBagConstraints          lGbc = new GridBagConstraints();

    lGbc.anchor = GridBagConstraints.CENTER;
    lGbc.fill   = GridBagConstraints.HORIZONTAL;
    lGbc.weightx = 20;
    lGbc.weighty = 10;

    add( mInput,   lGbc, 0, 0, 1, 1 );
    add( mOutput,  lGbc, 0, 1, 1, 1 );

    add( mTabChoice, lGbc, 0, 2, 1, 1 );

    lGbc.fill = GridBagConstraints.NONE;
    add( mInputTextField,  lGbc, 1, 0, 3, 1 );
    add( mOutputTextField, lGbc, 1, 1, 3, 1 );
    add( mTabs,   lGbc, 2, 2, 1, 1 );
    add( mSpaces, lGbc, 3, 2, 1, 1 );

    addWindowListener( new WindowAdapter()
                       {
                         public void windowClosing(
                         WindowEvent evt )
                         {
                           setVisible( false );
                           System.exit( 0 );
                         }
                       }
                     );

    setLocation( 100, 100 );
    setSize( 400, 100 );
}

  public void                   add(
  Component                     oComponent,
  GridBagConstraints            oGbc,
  int                           x,
  int                           y,
  int                           w,
  int                           h )
  {
    oGbc.gridx = x;
    oGbc.gridy = y;
    oGbc.gridwidth  = w;
    oGbc.gridheight = h;
    add( oComponent, oGbc );
  }

  public Choice                 addChoice()
  {
    Choice                      lChoice = new Choice();

    lChoice.addItemListener( this );

    return lChoice;
  }

  public Button                 addButton(
  String                        oName )
  {
    Button                      lButton = new Button( oName );

    lButton.addActionListener( this );

    return lButton;
  }

  public TextField              addTextField()
  {
    TextField                   lTextField = new TextField( "", 40 );

    lTextField.addTextListener( this );

    return lTextField;
  }

  public void                   ActionInput()
  {
    String                      lFilename;
    String                      lExtension;

    lFilename = SetFile( "*.*", "Input file", FileDialog.LOAD );
    if ( lFilename != null )
    {
      mInputTextField.setText( lFilename );
      mOutputTextField.setText( lFilename );
    }
  }

  private void                  CloseFiles()
  {
    if ( mInputFile != null )
    {
      try
      {
        mInputFile.close();
      }
      catch ( Exception e )
      {
        System.err.println( "Can't close Input file" );
      }
    }

    if ( mOutputFile != null )
    {
      try
      {
        mOutputFile.close();
      }
      catch ( Exception e )
      {
        System.err.println( "Can't close Output file" );
      }
    }

    mInputFile  = null;
    mOutputFile = null;
  }

  private String                OpenFiles(
  String                        oInputFilename,
  String                        oOutputFilename )
  {
    if ( ( oInputFilename == null      )
      || ( oInputFilename.equals( "" ) ) )
    {
      return " No Input file";
    }
    try
    {
      mInputFile = new BufferedReader(
                   new FileReader( oInputFilename ) );
    }
    catch ( Exception e )
    {
      return "Can't open Input file " + oInputFilename;
    }

    if ( ( oOutputFilename == null      )
      || ( oOutputFilename.equals( "" ) ) )
    {
      return " No Output file";
    }

    try
    {
      mOutputFile = new PrintWriter(
                    new FileWriter( oOutputFilename, false ) );
    }
    catch ( Exception e )
    {
      return "Can't open Output file " + oOutputFilename;
    }

    return null;
  }

  private String                SetFile(
  String                        oFilter,
  String                        oHeader,
  int                           iMode )
  {
    FileDialog                  lFd;
    String                      lFilename;

    lFd = new FileDialog( this      // mFrame
                        , oHeader
                        , iMode );
    lFd.setDirectory( mDirectory );
    lFd.setFile( oFilter );
    lFd.show();

    lFilename = lFd.getFile();
    if ( lFilename != null )
    {
      mDirectory = lFd.getDirectory();
      lFilename = mDirectory + lFilename;
    }

    return lFilename;
  }

  private void                  PrintDialog(
  boolean                       bDialog,
  Frame                         oFrame,
  String                        oHeader,
  String                        oText )
  {
    if ( bDialog )
    {
      new PbnDialog( oFrame, oHeader, oText );
    }
    else
    {
      System.out.println( oText );
    }
  }

  private String                LineDetab(
  String                        oInput )
  {
    int                         l_old = oInput.length();
    int                         i_tab = 0;
    char                        c;
    StringBuffer                lOutput = new StringBuffer( 80 );

    for ( int i_old = 0; i_old < l_old; i_old++ )
    {
      switch ( (c = oInput.charAt( i_old )) )
      {
      case '\t':
        for ( ; i_tab < mTabSize; i_tab++ )
        {
          lOutput.append( ' ' );
        }
        i_tab = 0;
        break;
      default:
        lOutput.append( c );
        if ( ++i_tab == mTabSize )
        {
          i_tab = 0;
        }
        break;
      }
    }

    return lOutput.toString();
  }

  private boolean               ExecuteDetab(
  BufferedReader                oInputFile,
  PrintWriter                   oOutputFile )
  {
    String                      lInput;
    String                      lOutput;
    int                         i_line = 0;

    try
    {
      while ( (lInput = mInputFile.readLine()) != null )
      {
        lOutput = LineDetab( lInput );
        mOutputFile.println( lOutput );
        i_line++;
      }
    }
    catch ( Exception e )
    {
      return false;
    }

    return true;
  }

  private String                LineEntab(
  String                        oInput )
  {
    int                         l_old = oInput.length();
    int                         i_tab = 0;
    int                         n_space = 0;
    char                        c;
    StringBuffer                lOutput = new StringBuffer( 80 );

    for ( int i_old = 0; i_old < l_old; i_old++ )
    {
      switch ( (c = oInput.charAt( i_old )) )
      {
      case ' ':
        n_space++;
        if ( ++i_tab == mTabSize )
        {
          i_tab = 0;
          lOutput.append( ( n_space == 1 ) ? ' ' : '\t' );
          n_space = 0;
        }
        break;
      case '\t':
        lOutput.append( '\t' );
        i_tab = 0;
        n_space = 0;
        break;
      default:
        for ( ; n_space > 0; n_space-- )
        {
          lOutput.append( ' ' );
        }
        lOutput.append( c );
        if ( ++i_tab == mTabSize )
        {
          i_tab = 0;
        }
        break;
      }
    }

    return lOutput.toString();
  }

  private boolean               ExecuteEntab(
  BufferedReader                oInputFile,
  PrintWriter                   oOutputFile )
  {
    String                      lInput;
    String                      lOutput;
    int                         i_line = 0;

    try
    {
      while ( (lInput = mInputFile.readLine()) != null )
      {
        lOutput = LineEntab( lInput );
        mOutputFile.println( lOutput );
        i_line++;
      }
    }
    catch ( Exception e )
    {
      return false;
    }

    return true;
  }

  private boolean               MyRename(
  String                        oSourceFilename,
  String                        oTargetFilename )
  {
    File                        lSourceFile = new File( oSourceFilename );
    File                        lTargetFile = new File( oTargetFilename );

    try
    {
      lTargetFile.delete();
      lSourceFile.renameTo( lTargetFile );
    }
    catch ( Exception e )
    {
      return false;
    }
    return true;
  }

  private boolean               ActionConvert(
  String                        oInputFilename,
  String                        oOutputFilename,
  boolean                       bEntab,
  boolean                       bDialog )
  {
    String                      lString;
    boolean                     bInIsOut = false;
    String                      lInIsOutName = "tab.tmp";
    boolean                     bOK = false;


    if ( ( oOutputFilename != null                   )
      && ( oInputFilename.equals( oOutputFilename ) ) )
    {
      bInIsOut = true;
      if ( lInIsOutName.equals( oOutputFilename ) )
      {
        lInIsOutName = "tab.tmq";
      }
      oOutputFilename = lInIsOutName;
    }

    lString = OpenFiles( oInputFilename
                       , oOutputFilename );
    if ( lString != null )
    {
      PrintDialog( bDialog, this, "Files", lString );
    }
    else
    {
      if ( bDialog )
      {
        this.setCursor( WAIT_CURSOR );
      }

      if ( bEntab )
      {
        bOK = ExecuteEntab( mInputFile, mOutputFile );
      }
      else
      {
        bOK = ExecuteDetab( mInputFile, mOutputFile );
      }

      if ( bDialog )
      {
        this.setCursor( DEFAULT_CURSOR );
      }
    }

    CloseFiles();
    if ( bOK && bInIsOut )
    { /*
       * Rename the temporary file to the Output file.
       */
      if ( ! MyRename( lInIsOutName, oInputFilename ) )
      {
        bOK = false;
        PrintDialog( bDialog, this, "File error"
                   , "Can't Output to Input file" );
      }
    }

    return bOK;
  }

  private void                  ActionConverts(
  String                        oInputFilename,
  boolean                       bEntab,
  boolean                       bDialog )
  {
    File                        lFile = new File( oInputFilename );
    String                      lPattern = lFile.getName();
    String                      lPath;
    File                        lDirectory;
    String []                   lFiles;
    String []                   lMatchedFiles;
    boolean                     bOK;

    try
    {
      lPath = lFile.getParent();
      lDirectory = new File( lPath );
      lFiles = lDirectory.list();
      if ( lFiles == null )
      {
        PrintDialog( bDialog, this, "Conversion error"
                   , "No files" );

        return;
      }
    }
    catch ( Exception e )
    {
      System.err.println( "Can't construct directory" );
      return;
    }

    lMatchedFiles = PatternMatch.MatchList( lFiles, lPattern );

    for ( int i = 0; i < lMatchedFiles.length; i++ )
    {
      File lConvertFile = new File( lDirectory, lMatchedFiles[ i ] );

      bOK = ActionConvert( lConvertFile.toString()
                         , lConvertFile.toString()
                         , bEntab
                         , bDialog );

      if ( ! bOK )
      {
        PrintDialog( bDialog, this, "Conversion error"
                   , "Can't execute conversion of file " + lMatchedFiles[ i ] );
      }
    }
  }

  public void                   actionPerformed(
  ActionEvent                   evt )
  {
    String                      lName = evt.getActionCommand();
    String                      lFilename;

    if ( lName.equals( "Input" ) )
    {
      ActionInput();
    }
    else
    if ( lName.equals( "Output" ) )
    {
      lFilename = SetFile( "*.*", "Output file", FileDialog.LOAD );
      if ( lFilename != null )
      {
        mOutputTextField.setText( lFilename );
      }
    }
    else
    if ( ( lName.equals( "Tabs"   ) )
      || ( lName.equals( "Spaces" ) ) )
    {
      String                    lInputFilename;
      lInputFilename = mInputTextField.getText();

      boolean                   bEntab = lName.equals( "Tabs" );
      boolean                   bOK;

      if ( ( lInputFilename.indexOf( "*" ) == -1 )
        && ( lInputFilename.indexOf( "?" ) == -1 ) )
      {
        bOK = ActionConvert( lInputFilename
                           , mOutputTextField.getText()
                           , bEntab
                           , true );
        if ( ! bOK )
        {
          PrintDialog( true, this, "File error", "Can't execute conversion" );
        }
        else
        {
          PrintDialog( true, this, "Conversion Result", "Ready" );
        }
      }
      else
      {
        ActionConverts( lInputFilename
                      , bEntab
                      , true );
        PrintDialog( true, this, "Conversion Result", "Ready" );
      }
    }
  }

  public void                   itemStateChanged(
  ItemEvent                     evt )
  {
    if ( evt.getStateChange() == ItemEvent.DESELECTED )
    {
      return;
    }

    String                      lItem = (String) evt.getItem();

    mTabSize = (int) java.lang.Integer.parseInt( lItem );
  }

  public void                   textValueChanged(
  TextEvent                     evt )
  {
  }

  public static void            main(
  String []                     args )
  {
    Frame                       lTab;

    lTab = new tab();
    lTab.show();
  }
}

class PbnDialog extends Dialog
{
  public                        PbnDialog(
  Frame                         oParent,
  String                        oHeader,
  String                        oText )
  {
    super( oParent, oHeader, true );

    Panel                       lPanel = new Panel();
    int                         NrLines;
    int                         Index;

    for ( NrLines = 0; ; NrLines++ )
    {
      Index = oText.indexOf( "\n" );
      if ( Index < 0 )
      {
        lPanel.add( new Label( oText ) );
        break;
      }
      lPanel.add( new Label( oText.substring( 0, Index ) ) );
      oText = oText.substring( Index+1 );
    }
    add( lPanel, "Center" );

    Button                      mOk = new Button( "OK" );
    lPanel = new Panel();
    lPanel.add( mOk );
    add( lPanel, "South" );

    mOk.addActionListener( new ActionListener()
                           {
                             public void actionPerformed(
                             ActionEvent evt )
                             {
                               setVisible( false );
                             }
                           }
                         );

        addWindowListener( new WindowAdapter()
                           {
                             public void windowClosing(
                             WindowEvent evt )
                             {
                               setVisible( false );
                             }
                           }
                         );

    setLocation( 150, 150 );
    setSize( 300, 120 + NrLines * 30 );
    show();
  }
}
